from ._funcs import (add_paths_to_system, init_django_app, log_to_console)
from .django_migration import (DjangoMigration)
from ._constants import *

__all__ = [
	"add_paths_to_system", "init_django_app", "log_to_console", "DjangoMigration",
]



import logging
import os

from django.core.management import call_command
from django.db import connections

from ._constants import DJANGO_MIGRATION_DB_VENDOR_CONFIG
from ._dto import DbVendorConfig
from ._funcs import log_to_console


class DjangoMigration():
	migration_dir: str
	__vendor_config: dict
	__database_set: set = set()
	__remove_tables_from_database_set: set = set()
	__apps_set: set = set()
	__is_dev_env: bool = False
	__remove_migration_files: bool = False
	__migrations_only: bool = False

	def __execute_cmd(self, *args):
		log_to_console(msg=f"executing {' '.join(args if len(args) > 0 else 'all')} ", format=True)
		call_command(*args)

	def __db_query(self, db_connection, *queries):
		if db_connection and len(queries) > 0:
			with db_connection.cursor() as cursor:
				_result = []
				for query in queries:
					print(f"Executing Query = {query}\n")
					query_result = cursor.execute(query)
					if query.lstrip(' ').lower().startswith(('select', 'show')):
						_result.append(cursor.fetchall())
					else:
						_result.append(query_result)
				return _result[0] if len(_result) == 1 else _result

	def __init__(self, migration_dir, **kwargs):
		self.migration_dir = migration_dir
		self.__vendor_config = DJANGO_MIGRATION_DB_VENDOR_CONFIG
		self.__is_dev_env = kwargs.get('is_dev_env') == True
		self.__remove_migration_files = kwargs.get('remove_files') == True

	def __get_db_vendor_config(self, db_connection) -> DbVendorConfig:
		if db_connection and hasattr(db_connection, 'vendor'):
			return self.__vendor_config.get(getattr(db_connection, 'vendor'))
		return None

	def __remove_files(self):
		if self.__is_dev_env or self.__remove_migration_files:
			log_to_console(msg="Removing Migration Files.", format=True)
			for miration_file in os.listdir(self.migration_dir):
				if miration_file not in ['__init__.py', '__pycache__']:
					os.remove(os.path.join(self.migration_dir, miration_file))
					log_to_console(miration_file)

	def __remove_tables(self):
		if self.__is_dev_env:
			for db in list(self.__remove_tables_from_database_set):
				con = connections[db]
				db_config = self.__get_db_vendor_config(db_connection=con)
				if db_config:
					log_to_console(f"Removing Database Tables From {db}", format=True)
					table_list = self.__db_query(con, db_config.query_to_list_tables)
					queries_to_drop_tables = db_config.queries_to_drop_multiple_tables(table_list, )

					self.__db_query(con, *[*queries_to_drop_tables])
		else:
			logging.warning(msg=f"Removing Tables only allowed in Dev Enviroment")

	def __build_migrations(self):
		app_list = list(self.__apps_set)
		return self.__execute_cmd(*["makemigrations", *app_list])

	def __migrate_databases(self):
		if not self.__migrations_only:
			for db in list(self.__database_set):
				self.__execute_cmd("migrate", "--database", db)
		else:
			logging.warning(msg="Migrations Only is Set To True", format=False)

	def configure(self, db_list: list, apps_list: list, **kwargs):

		self.__migrations_only = kwargs.get('migrations_only') == True

		if isinstance(db_list, list) and len(db_list) > 0:
			for db in db_list:
				self.__database_set.add(db)

		if isinstance(apps_list, list) and len(apps_list) > 0:
			for app in apps_list:
				self.__apps_set.add(app)

		remove_tables_from_databases = kwargs.get('remove_tables_from_databases')
		if isinstance(remove_tables_from_databases, list) and len(remove_tables_from_databases) > 0:
			for database in remove_tables_from_databases:
				self.__remove_tables_from_database_set.add(database)

		return self

	def run(self, **kwargs):
		try:
			self.__remove_files()

			if kwargs.get('remove_tables') is True:
				self.__remove_tables()

			self.__build_migrations()
			self.__migrate_databases()
		except Exception as e:
			logging.exception(e)



from ._schema import InputField
from ._decorators import ValidationSchema


class DefaultRules:
	date = InputField.Date(regex="^\d{4}\-(0?[1-9]|1[012])\-(0?[1-9]|[12][0-9]|3[01])$")
	pk = InputField.String(required=True)
	gender = InputField.String(allowed=['M', 'F', 'O'])

	@staticmethod
	def pk_validation_rule(pk_name: str):
		return ValidationSchema(**{pk_name: InputField.String(required=True)})





class ApiRequestClient:
	_url: str = ''
	__headers: dict = dict()
	_request_method: str = 'GET'

	def get(self, data: dict = None, url_params: dict = None, **opt):
		self.__set_request_url(url=opt.get('endpoint'), params=url_params)
		return self.__make_request(method='GET', data=data, **opt)

	def post(self, data: dict = None, url_params: dict = None, **opt):
		self.__set_request_url(url=opt.get('endpoint'), params=url_params)
		return self.__make_request(method='POST', data=data, **opt)

	def put(self, data: dict = None, url_params: dict = None, **opt):
		self.__set_request_url(url=opt.get('endpoint'), params=url_params)
		return self.__make_request(method='PUT', data=data, **opt)

	def delete(self, data: dict = None, url_params: dict = None, **opt):
		self.__set_request_url(url=opt.get('endpoint'), params=url_params)
		return self.__make_request(method='DELETE', data=data, **opt)

	def request(self, data: dict = None, url_params: dict = None, **opt):
		self.__set_request_url(url=opt.get('endpoint'), params=url_params)
		return self.__make_request(method=self._request_method, data=data, **opt)

	def __init__(self, url, **kwargs):
		self.__set_request_url(url=url, params={})
		if kwargs.get('auth_token'):
			self.__headers['Authorization'] = f"Bearer {kwargs.get('auth_token')}"

		if kwargs.get('method'):
			self._request_method = kwargs.get('method')

	def __str__(self):
		return f"<{self.__class__.__name__} {self._request_method.upper()} '{self._url}'>"

	def __set_request_url(self, url: str = None, params: dict = None):
		params_list = params.values() if isinstance(params, dict) else []
		url = self._url if url is None else url
		self._url = f"{url}/{'/'.join([str(i) for i in params_list])}".rstrip('/')

	def __make_request(self, method='post', data: dict = None, **opt) -> dict:
		from requests import api
		data = dict() if data is None else data
		if opt.get('headers'):
			self.__headers = dict(**self.__headers, **opt.get('headers'))

		api_response = api.request(url=self._url, method=method, data=data, headers=self.__headers)

		if api_response.headers.get('Content-Type') == "application/json":
			return api_response.json()
		return dict(body=api_response.content.decode())



from msb_validation import DefaultRules, validate_against, InvalidPayloadException
from ._api_view import ApiView
from ._constants import (CRUD_MAPPINGS, BULK_MAPPINGS, CrudActions, CRUD_URL_PK_NAME)
from ._exceptions import CrudApiExceptions
from ..services import ApiService
from ..wrappers import (RestResponse)


class ApiViewset(ApiView):
	service_class: ApiService = None
	validation_schema_class = None
	pk_name: str = CRUD_URL_PK_NAME

	def __validate_crud_request(self, action: str, inp=None, unknown=True, bulk=False, rule=None):
		rule = self.validation_schema_class.get(action, default={}) if rule is None else rule
		if (validation_errors := validate_against(schema=rule, inp=inp, unknown=unknown, bulk=bulk)) is not None:
			raise InvalidPayloadException(errors=validation_errors)

	@classmethod
	def crud_routes(cls, actions: list = None, **kwargs):
		try:
			from django.urls import path, include

			pk_name = cls.pk_name or CRUD_URL_PK_NAME
			allowed_actions = actions if isinstance(actions, list) else []
			single_operation_mappings, bulk_operation_mappings = (dict(), dict())

			for single_mapping, bulk_mapping in zip(CRUD_MAPPINGS.keys(), BULK_MAPPINGS.keys()):
				if single_mapping in allowed_actions and single_mapping not in [CrudActions.create]:
					single_operation_mappings.update(CRUD_MAPPINGS.get(single_mapping))

				if bulk_mapping in allowed_actions:
					bulk_operation_mappings.update(BULK_MAPPINGS.get(bulk_mapping))

			return [
				path("", cls.as_view(actions=bulk_operation_mappings)),
				path(f"/<str:{pk_name}>", cls.as_view(actions=single_operation_mappings))
			]

		except Exception as e:
			return None

	def create(self, *args, **kwargs) -> RestResponse:
		try:
			is_bulk = not isinstance(self.payload, dict)
			self.__validate_crud_request(CrudActions.create, inp=self.payload, unknown=False, bulk=is_bulk)
			_create_payload = [self.payload] if not is_bulk else self.payload

			if not (result := self.service_class.create(*_create_payload)):
				raise CrudApiExceptions.CreateOperationFailed
			return self.api_response.success()

		except Exception as e:
			return self.api_response.exception(e)

	def retrieve(self, *args, **kwargs) -> RestResponse:
		try:
			self.__validate_crud_request(CrudActions.retrieve, inp=kwargs)
			_pk = kwargs.get(self.pk_name)
			result = self.service_class.retrieve(pk=_pk)
			return self.api_response.success(data=self.serializer(data=[result]))
		except Exception as e:
			return self.api_response.exception(e=e)

	def list(self, *args, **kwargs) -> RestResponse:
		try:

			result = self.service_class.list()
			return self.api_response.success(data=self.serializer(data=result))
		except Exception as e:
			return self.api_response.exception(e=e)

	def update(self, *args, **kwargs) -> RestResponse:
		try:
			_rule = DefaultRules.pk_validation_rule(self.pk_name)
			_pk = kwargs.get(self.pk_name)
			_update_data = self.payload
			self.__validate_crud_request(CrudActions.update, inp=kwargs, rule=_rule)
			self.__validate_crud_request(CrudActions.update, inp=_update_data)
			result = self.service_class.update(pk=_pk, **_update_data)
			if not (result):
				raise CrudApiExceptions.UpdateOperationFailed
			return self.api_response.success()
		except Exception as e:
			return self.api_response.exception(e=e)

	def bulk_update(self, *args, **kwargs) -> RestResponse:
		try:
			_payload_list = [self.payload] if isinstance(self.payload, dict) else self.payload
			self.__validate_crud_request(CrudActions.update, inp=_payload_list, bulk=True)
			for _payload in _payload_list:
				if (_pk := _payload.get(self.pk_name)) is not None:
					del _payload[self.pk_name]
					self.service_class.update(pk=_pk, **_payload)
			return self.api_response.success()
		except Exception as e:
			return self.api_response.exception(e=e)

	def delete(self, *args, **kwargs) -> RestResponse:
		try:
			_rule = DefaultRules.pk_validation_rule(self.pk_name)
			_pk = kwargs.get(self.pk_name)
			self.__validate_crud_request(CrudActions.delete, inp=kwargs, rule=_rule)
			if not (result := self.service_class.delete(pk=_pk)):
				raise CrudApiExceptions.DeleteOperationFailed
			return self.api_response.success()
		except Exception as e:
			return self.api_response.exception(e=e)

	def bulk_delete(self, *args, **kwargs) -> RestResponse:
		try:
			_rule = DefaultRules.pk_validation_rule(self.pk_name)
			_payload = [self.payload] if isinstance(self.payload, dict) else self.payload
			self.__validate_crud_request(CrudActions.delete, inp=_payload, rule=_rule, bulk=True)
			for pk_details in _payload:
				_pk = pk_details.get(self.pk_name)
				self.service_class.delete(pk=_pk)
			return self.api_response.success()
		except Exception as e:
			return self.api_response.exception(e=e)



from django.core.serializers import serialize
from django.db import models as Models

from msb_cipher import Cipher
from ._model_manager import MsbModelManager
from .._constants import (COLUMN_NAME_DELETED, COLUMN_NAME_DELETED_BY)


class MsbModel(Models.Model):
	_private_fields = []
	_identifier_field = ""

	class Meta:
		abstract = True

	@property
	def related_fields(self):
		fields = []
		for field in self._meta.fields:
			if field.get_internal_type() in ['ForeignKey']:
				fields.append(field.name)
		return fields

	def dict(self, encrypted=True):
		try:
			return {
				k: v if (k not in [
					self._meta.pk.attname, *self._private_fields
				] or not encrypted) else Cipher.encrypt(v)
				for k, v in super().__dict__.items()
				if not k.startswith('__') and not k.startswith('_') and not callable(k)
			}

		except Exception:
			return dict()

	@property
	def serialized(self):
		return serialize('python', [self])

	def delete(self, deleted_by=None, using=None, keep_parents=False):
		if hasattr(self, COLUMN_NAME_DELETED):
			self.__setattr__(COLUMN_NAME_DELETED, True)

		if hasattr(self, COLUMN_NAME_DELETED_BY):
			self.__setattr__(COLUMN_NAME_DELETED_BY, deleted_by)
		self.save()
		return True


	def __str__(self):
		pk_field = f"[{getattr(self, self._meta.pk.attname)}]" if hasattr(self._meta, "pk") else ""
		idf_field = f"{getattr(self, self._identifier_field)}" if hasattr(self, self._identifier_field) else ""
		return f"<{self.__class__.__name__} {pk_field}: {idf_field}>"

	def __unicode__(self):
		return self.__str__()

	@property
	def _query(self):
		return self.objects

	objects = MsbModelManager()



CRUD_URL_PK = '/<str:pk>'
CRUD_URL_PK_NAME = "pk"
MAX_FETCH_LIMIT = 100
DEFAULT_LOGGER_NAME = 'root'

class REQUEST_METHODS:
	POST = "post"
	PUT = "put"
	GET = "get"
	DELETE = "delete"

	ALL = [POST, PUT, GET, DELETE]


class CrudActions:
	create = 'create'
	retrieve = 'retrieve'
	list = 'list'

	update = 'update'
	bulk_update = 'bulk_update'

	delete = 'delete'
	bulk_delete = 'bulk_delete'
	all = [create, retrieve, update, delete, bulk_delete, bulk_update, list]


CRUD_MAPPINGS = {
	CrudActions.retrieve: {REQUEST_METHODS.GET: CrudActions.retrieve},
	CrudActions.update: {REQUEST_METHODS.PUT: CrudActions.update},
	CrudActions.delete: {REQUEST_METHODS.DELETE: CrudActions.delete},
	CrudActions.create: {REQUEST_METHODS.POST: CrudActions.create},

}

BULK_MAPPINGS = {
	CrudActions.bulk_delete: {REQUEST_METHODS.DELETE: CrudActions.bulk_delete},
	CrudActions.bulk_update: {REQUEST_METHODS.PUT: CrudActions.bulk_update},
	CrudActions.list: {REQUEST_METHODS.GET: CrudActions.list},
	CrudActions.create: {REQUEST_METHODS.POST: CrudActions.create},

}



import re

ENV_LOAD_STATUS_KEY = 'ENV_CONFIG_LOADED'

DOT_ENV_FILE = ".env"

LOCAL_CONFIG_FILE_PREFIX = "local"

ENVIRONMENT_VARIABLE_NAME = "ENVIRONMENT"

DEBUG_VARIABLE_NAME = "ENVIRONMENT"

DEBUG_VARIABLE_VALUE = False



from rest_framework import (viewsets, serializers)

from msb_auth import TokenUser
from ..wrappers import (ApiResponse, RequestInfo, RequestHeaders)
from ._constants import DEFAULT_LOGGER_NAME


def api_details(request=None, ver='', name=''):
	return ApiResponse.success(
		data=dict(method=request.method, version=ver, name=name)
	)


class ApiView(viewsets.GenericViewSet):
	permission_classes = ()
	serializer_class = serializers.Serializer
	default_logger: str = DEFAULT_LOGGER_NAME

	"""
	TODO : implement following code :
	
	@staticmethod
	@renderer_classes((JSONRenderer))
	def my_exception_handler(exc, context):
		# response = exception_handler(exc, context)  # <-- this is the default exception_handler
		import logging
		logging.exception(exc)
		return ApiResponse.internal_server_error()

	def get_exception_handler(self):
		return self.my_exception_handler
	"""

	@property
	def request_info(self) -> RequestInfo:
		return RequestInfo(meta=self.request.META)

	@property
	def request_headers(self) -> RequestHeaders:
		return RequestHeaders(headers=self.request.headers)

	@property
	def user(self) -> TokenUser:
		return self.request.user

	@property
	def payload(self) -> dict | list:
		return self.request.data if type(self.request.data) in [list, dict] else {}

	@property
	def logger(self):
		import logging
		return logging.getLogger(self.default_logger)

	@property
	def api_response(self):
		return ApiResponse

	def serializer(self, data: list = None):
		from django.db.models import QuerySet
		return (
			[item.dict() if hasattr(item, 'dict') else item for item in data]
		) if (type(data) in [list, QuerySet]) else []



from django.db import models

from ._model import MsbModel
from ._model import MsbModelManager
from ._fields import EncryptedString


class ConfigurationModelManager(MsbModelManager):
	pass


# Configuration model
class Configuration(MsbModel):
	# meta data
	class Meta:
		abstract = True
		indexes = [models.Index(fields=['name', 'key'])]

	# Model fields
	name = models.CharField(max_length=100, db_column='name')
	key = models.CharField(max_length=255, db_column='key')
	value = EncryptedString(db_column='value')

	# assign a custom manager to the model
	objects = ConfigurationModelManager()

	def __str__(self):
		return self.name

	def __unicode__(self):
		return self.name



import os
import sys


def add_paths_to_system(*path_list):
	for path in path_list:
		sys.path.append(path)


def init_django_app(settings_dir: str, **kwargs):
	import django
	os.environ.setdefault("DJANGO_SETTINGS_MODULE", f"{settings_dir}.settings")
	os.environ.setdefault("PYTHONUNBUFFERED", 1)

	if isinstance(kwargs.get('sys_pathlist'), list):
		add_paths_to_system(*kwargs.get('sys_pathlist'))
	django.setup()


def log_to_console(msg, format=False):
	_log_message = f"\n{f'[ {msg} ] ' :*^100}" if format else f"LOG : {msg}"
	return print(_log_message)



from msb_exceptions import ApiException


class CrudApiExceptions:
	class CreateOperationFailed(ApiException):
		_message = "Failed to create the requested data."

	class DeleteOperationFailed(ApiException):
		_message = "Failed to delete the requested data."


	class UpdateOperationFailed(ApiException):
		_message = "Failed to update the requested data."


	class RetrieveOperationFailed(ApiException):
		_message = "Failed to retrieve the requested data."


class AuthUser:
	__auth_data = dict()

	def __getitem__(self, item):
		return self.__auth_data.get(item)

	def __init__(self, type: str = None, username: str = None, userid: str = None, **kwargs):
		self.__auth_data['auth_type'] = type
		self.__auth_data['userid'] = userid
		self.__auth_data['username'] = username
		self.__auth_data = dict(**self.__auth_data, **kwargs)
		self.__auth_data['is_authenticated'] = (
			username is not None and userid is not None
		)

	@property
	def id(self):
		return self.__auth_data.get('userid')

	@property
	def user(self):
		return self.id

	@property
	def username(self):
		return self.__auth_data.get('username')

	@property
	def is_authenticated(self) -> bool:
		return self.__auth_data.get('is_authenticated')

	@property
	def auth_data(self) -> dict:
		return self.__auth_data



from django.utils.functional import cached_property


class TokenUser:
	token: dict
	__auth_data: dict

	def __init__(self, tokendata: dict = None):
		self.token = tokendata if tokendata else dict()

	def __str__(self):
		return f"<TokenUser,id={self.userid},user={self.username}>"

	def __hash__(self):
		return hash(self.use)

	def __getattr__(self, attr):
		"""This acts as a backup attribute getter for custom claims defined in Token serializers."""
		return self.token.get(attr, None)

	def set_validation_status(self, status: bool = False):
		self.token['is_valid'] = status

	def get(self, key: str = '', default=None):
		val = self.__auth_data.get(key)
		return val if val is not None else None

	@cached_property
	def userid(self):
		return self.get('userid')

	@cached_property
	def username(self):
		return self.get('username')

	@cached_property
	def auth_type(self):
		return self.get('auth_type')

	@cached_property
	def is_admin(self):
		return self.get("is_admin")

	@property
	def is_valid(self):
		return self.token.get('is_valid', False)

	@property
	def is_authenticated(self):
		return self.get('is_authenticated', False) and self.is_valid

	@property
	def role(self):
		return self.get('role')

	@property
	def id(self):
		return self.get('employee_number')

	def __set_auth_data(self):
		self.__auth_data = self.token.get('userdata').get('auth_data') if self.token else dict()




DEFAULT_DATE = "%Y-%m-%d"
DEFAULT_TIME = "%H:%M:%S"
TIMESTAMP = f"{DEFAULT_DATE} {DEFAULT_TIME}"

DEFAULT_DATE_SLASHED = '%Y/%m/%d'
TIMESTAMP_SLASHED = f"{DEFAULT_DATE_SLASHED} {DEFAULT_TIME}"
HUMAN_FORMAT = "%a %d %b %y, %I:%M:%S %p"
DEFAULT_TIMEZONE = 'Asia/Kolkata'


from ._model import (MsbModel)
from ._model_manager import (MsbModelManager,)
from ._metafields import ( MsbModelMetaFields)
from .config_model import (ConfigurationModelManager, Configuration)
from .logging_models import (SystemLogModel, LoggingModel, LoggingModelManager, )
from ._fields import (
	EncryptedBool, EncryptedInteger, EncryptedString,
	EncryptedFloat, EncryptedPrimaryKey
)

__all__ = [
	'Configuration',
	'ConfigurationModelManager',
	'MsbModel',
	'MsbModelManager',
	'EncryptedBool',
	'EncryptedInteger',
	'EncryptedString',
	'EncryptedFloat',
	'EncryptedPrimaryKey',
	'LoggingModelManager',
	'SystemLogModel',
]



from msb_cipher import Cipher
from msb_dataclasses import Singleton
from msb_dataclasses.search_parameter import SearchParameter
from msb_db.models import MsbModel
from msb_exceptions import CustomExceptionHandler
from ._exceptions import ApiServiceExceptions


class ApiService(CustomExceptionHandler, metaclass=Singleton):
	db_model = MsbModel

	@property
	def cipher(self):
		return Cipher

	def __decrypt(self, data):
		return Cipher.decrypt(data=data)

	def __init__(self):
		if self.db_model is None:
			raise ApiServiceExceptions.InvalidDatabaseModel

	def search(self, params: SearchParameter):
		try:
			model_query = self.db_model.objects.only(*params.fields).filter(**params.filters)
			if params.order_field is not None:
				model_query.order_by(params.order_field)
			return model_query.all()[params.offset:params.offset + params.limit]
		except Exception:
			return False

	def create(self, *model_data_list):
		try:
			create_status = False
			if len(model_data_list) <= 0:
				raise ApiServiceExceptions.InvalidDataForCreateOperation

			if len(model_data_list) == 1:
				_model = (self.db_model(**model_data_list[0])).save()
				create_status = True
			else:
				create_status = self.db_model.objects.bulk_create(
					[self.db_model(**model_data) for model_data in model_data_list]
				)
			return create_status
		except Exception:
			raise ApiServiceExceptions.CreateOperationFailed

	def retrieve(self, pk=None):
		try:
			if pk is None:
				raise ApiServiceExceptions.InvalidPk
			return self.db_model.objects.get(pk=pk)
		except Exception:
			raise ApiServiceExceptions.RetrieveOperationFailed

	def list(self, fields: list = None, filters: dict = None):
		try:
			model_objects = self.db_model.objects
			if isinstance(fields, list) and len(fields) > 0:
				model_objects.only(*fields)

			if isinstance(filters, dict) and len(filters.keys()) > 0:
				model_objects.filter(**filters)
			offset, limit = (0, 50)
			data_list = model_objects.all()[offset:limit]
			return data_list
		except Exception:
			raise ApiServiceExceptions.ListOperationFailed

	def update(self, pk=None, **model_data):
		try:
			if (pk := self.__decrypt(pk)) is None:
				raise ApiServiceExceptions.InvalidPk
			model_object = self.db_model.objects.filter(pk=pk)
			if model_object and not (status := model_object.update(**model_data)):
				raise ApiServiceExceptions.UpdateOperationFailed
			return bool(status)
		except Exception:
			raise ApiServiceExceptions.UpdateOperationFailed

	def delete(self, pk=None):
		try:
			if (pk := self.__decrypt(pk)) is None:
				raise ApiServiceExceptions.InvalidPk
			model_object = self.db_model.objects.get(pk=pk)

			if model_object and not (status := model_object.delete()):
				raise ApiServiceExceptions.DeleteOperationFailed
			return status
		except Exception:
			raise ApiServiceExceptions.DeleteOperationFailed



from ._client import ApiRequestClient


class ApiEndpoint():
	__version_text = ''
	_url: str = ''
	_method: str = 'GET'

	def __str__(self):
		return f"<{self.__class__.__name__} {self._method.upper()} '{self._url}'>"

	def __init__(self, url: str = None, method: str = None, **kwargs):
		if kwargs.get('version_text'):
			self.__version_text = kwargs.get('version_text')
		self._url = url.rstrip('/')
		self._method = method

		self._init_data = kwargs

	def __versioned_api_request(self, version=1) -> ApiRequestClient:
		self._url = self._url.replace(self.__version_text, f'v{version}')
		return ApiRequestClient(url=self._url, method=self._method, **self._init_data)

	@property
	def v1(self) -> ApiRequestClient:
		return self.__versioned_api_request(version=1)

	@property
	def v2(self) -> ApiRequestClient:
		return self.__versioned_api_request(version=2)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['msb_apis',
 'msb_apis.services',
 'msb_apis.views',
 'msb_apis.wrappers',
 'msb_auth',
 'msb_cipher',
 'msb_config',
 'msb_dataclasses',
 'msb_datetime',
 'msb_db',
 'msb_db.models',
 'msb_devtools',
 'msb_exceptions',
 'msb_ext',
 'msb_files',
 'msb_files.core',
 'msb_logging',
 'msb_testing',
 'msb_validation']

package_data = \
{'': ['*']}

install_requires = \
['cerberus>=1.3.4,<2.0.0',
 'cffi>=1.15.1,<2.0.0',
 'cryptography>=38.0.3,<39.0.0',
 'django>=4.1.3,<5.0.0',
 'djangorestframework-simplejwt>=5.2.2,<6.0.0',
 'djangorestframework>=3.14.0,<4.0.0',
 'pandas>=1.5.1,<2.0.0',
 'pdf2docx>=0.5.6,<0.6.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'python-dotenv>=0.21.0,<0.22.0',
 'requests>=2.28.1,<3.0.0',
 'xhtml2pdf>=0.2.8,<0.3.0']

setup_kwargs = {
    'name': 'hrin-msb',
    'version': '0.1.41',
    'description': '',
    'long_description': '# hrin-msb\n\n## Pre-requisites for setup\n1. `pip install poetry`\n\n## How To Build\n\n1. `poetry build`\n2. `poetry config http-basic.pypi __token__ <access-token>`\n3. `poetry publish`\n\n\n# Change Log\n ### Version 0.1.1\n\n ### Version 0.1.2\n\n ### Version 0.1.3\n\n1.  Default serializer added to ApiView\n2. fixed incorrect import in _validators.py\n3. fixed msb_database_router\n4. fixed Config.is_local_env() not working\n5. moved devscripts -> devtools\n6. File Utils Added to utils/files\n7. "app_label" removed from "TestConfig" & "ApiTest" Classes\n8. Fixed Bug : \'LoggingModelManager\' object has no attribute \'_queryset_class\'\n9. Fixed : Logging Model not showing any records\n10. Fixed : str method for base model, & removed current_timestamp method from base model\n\n ### Version 0.1.4\n1. Fixed : ModuleNotFoundError: No module named \'pdf2docx\'\n2. Renamed “FileGenerator“ => “FileFactory”,\n3. Add `create_` Prefix in FileFactory methods\n4. Renamed MsbMetaModel -> MsbModelMetaFields\n5. Added validation decorators, and fixed bulk validation issuses\n6. Modified Logging Configuration Files\n7. removed utils package\n8. moved msb_core.wrappers.exceptions -> msb_exceptions\n9. Fixed : Base ApiViews and Crud Routes\n10. Searchparameter class refactored, search method added in ApiService Class\n\n### Version 0.1.41\n1. Fixed: Crud operations not working with encrypted id\'s\n2. Package dependencies updated\n3. Validator test cases refactored ',
    'author': 'Prakash Mishra',
    'author_email': 'prakash.mishra@intimetec.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from ._endpoint import ApiEndpoint
from ._request import (RequestHeaders, RequestInfo)
from ._client import ApiRequestClient
from ._response import (ApiResponse,RestResponse)
from ._host import ApiHost

